// ---------- c 2020 by DC5ZM -------------------------------------------------------------

// Use this sketch for SG_Lab PA version V3 with preamp  
// The calibration data for DRV, FWD and REV are based on the measured values ​​provided by SG.Lab
// PCB for UNO and NANO are available by weber.r1@t-online.de
// Please inquire by email before ordering

// ---------------------------------------------- include libraries --------------------------------------------------
#include <OneWire.h> 
#include <DallasTemperature.h>
#include <LiquidCrystal.h>
// ---------------------------------------------- define constants ---------------------------------------------------
LiquidCrystal lcd(8,9,4,5,6,7 ); //allocate uno pins to LCD diplay pins(rs,enable,DB4,DB5,DB6,DB7)    

#define ONE_WIRE_BUS 12    //  Data wire is plugged into pin 12 on the Arduino 
#define TEMPERATURE_PRECISION 13
#define FAN_PIN 11
#define NO  0
#define YES 1


// ************************************************************** variables to be changed for calibration *********
// use   https://valdivia.staff.jade-hs.de/interpol.html           for calculation of polygon coefficients        *
//                                                                   formula used:   y = a +b*x + c*x*x           *
float    a_fwd, b_fwd, c_fwd, a_ref, b_ref, c_ref, a_drv = 0 , b_drv = 0, c_drv = 0;;     // polygon coefficients *
//                               calibration values ​​have been shifted to the function "menu_directional_couplers" *  
byte     temp_fan_on = 50, voltmeter_full_scale = 30;                // fan on at overtemperature, adjust att. R1 *
// ****************************************************************************************************************


unsigned long time;   
uint16_t temp_integer;    // uint16_t = 16 bit integer

char     temp_string[8], power_fwd_string[8], power_ref_string[8], power_drv_string[8], swr_string[8]; 
char     pa_voltage_string[8], temp_fan_on_string[8];

float    reflexion_coefficient, swr , power_fwd, power_ref, power_drv, voltage_fwd,voltage_ref, voltage_drv;
float    voltage_fwd_peak =0,voltage_ref_peak =0, voltage_drv_peak =0; 
float    temperature, tmp_old = 0, pa_voltage;

byte     temp_byte, temp_bar_max = 100, iii=0;
byte     menu = 1, temp_fan_on_marker;
boolean  fwd_too_low = NO, ref_too_low = NO, fan_active = NO, error_swr = NO;

OneWire oneWire(ONE_WIRE_BUS); // Setup a oneWire instance to communicate with any OneWire devices  
DallasTemperature sensors(&oneWire); // Pass oneWire reference to Dallas Temperature

// ###################################################################################################################
void setup(void) 
{ 
  lcd.begin(16, 2);  // display with 16 colum and 2 lines
  sensors.begin();   // Start up the library 
  pinMode(FAN_PIN, OUTPUT);
  
  lcd.print("SG-Lab MONITOR 3");
  lcd.setCursor(0,1);  // colum, line nr. 
  delay(1000);
  lcd.print("c 2020 ");
  delay(200);
    lcd.print("by  ");
  delay(200);
    lcd.print("DC5ZM");
  delay(1000);
  lcd.clear();  



  Serial.begin(9600);                   // ############################################################++++++++++++++++++++++++++++

  
} 
// ###################################################################################################################
void loop(void) 
{   
  select_menu();
  read_temperature();  
}
// ###################################################################################################################
void select_menu()
{  
 if(analogRead(A0) < 1000 ) menu++;                        // if any of 5 keys pressed change menu
  while(analogRead(A0) < 1000);                            // wait for key released
  if(menu  > 3) menu = 1;
  if(menu == 1) menu_directional_couplers();
  if(menu == 2) menu_temperature();
  if(menu == 3) menu_supply_fan(); 
}
// -------------------------------------------------------------------------------------------------------------------
void menu_directional_couplers()                                               // formula used:   y = a +b*x + c*x*x
{   voltage_fwd_peak = 0;
    voltage_ref_peak = 0;
    voltage_drv_peak = 0;

      
  for(iii=0; iii<20; iii++)                                     // Take 20 samples and save the highest value
  { voltage_fwd = analogRead(A1);                                                 // test analog input / 327 =  1.6V 
    if(voltage_fwd < 327) {a_fwd = 0; b_fwd = -0.1493; c_fwd = 1.265;}            // polygon coefficiets for <  1.6V            
    else {a_fwd = 0.2611, b_fwd = -1.374, c_fwd = 1.822;}                         // polygon coefficiets for >= 1.6V 
//    voltage_fwd = analogRead(A1);                                                 // read analog input

  
    voltage_ref = analogRead(A2);                                                 // test analog input A2
    if(voltage_ref < 327) {a_ref = 0; b_ref = -0.1493; c_ref = 1.265;} 
    else {a_ref = 0.2611, b_ref = -1.374, c_ref = 1.822;}
//    voltage_ref = analogRead(A2);       

                                                           

    voltage_drv = analogRead(A3);                                                 // read analog input A3
    a_drv = 0 ; b_drv = 1.4; c_drv = 0.316;                                      // polygon coefficiets for DRV

    if(voltage_fwd > voltage_fwd_peak) voltage_fwd_peak = voltage_fwd;           // safe the peak of 10 measurements
    if(voltage_ref > voltage_ref_peak) voltage_ref_peak = voltage_ref;  
    if(voltage_drv > voltage_drv_peak) voltage_drv_peak = voltage_drv; 
  }
  voltage_fwd = voltage_fwd_peak;                                                // use peak voltage for processing
  voltage_ref = voltage_ref_peak;
  voltage_drv = voltage_drv_peak;


   

// ---------------------------------------- calculate coupler values -------------------------------------------------
  voltage_fwd = voltage_fwd*5/1023;                                       // normalize measured value FWD to 5V
  voltage_ref = voltage_ref*5/1023;                                             // ... REF to 5V
  voltage_drv = voltage_drv*5/1023;


Serial.print("voltage_ref = ");   // ############################################################################???????????????????????????????
Serial.println(voltage_ref);   // ############################################################################???????????????????????????????

  

  power_fwd = a_fwd + b_fwd*voltage_fwd + c_fwd*voltage_fwd*voltage_fwd;    // polygon approximatiion equation fwd
    if(power_fwd < 0) power_fwd = 0;                                          // avoid negativ values    

  power_ref = a_ref + b_ref*voltage_ref + c_ref*voltage_ref*voltage_ref;    // polygon approximatiion equation ref
    if(power_ref < 0) power_ref = 0; 


Serial.print("power_ref = ");   // ############################################################################???????????????????????????????
Serial.println(power_ref);   // ############################################################################???????????????????????????????

    

  if(power_fwd <= power_ref) error_swr = YES;    

  power_drv = a_drv + b_drv*voltage_drv + c_drv*voltage_drv*voltage_drv;    // polygon approximatiion equation drv
    if(power_drv < 0) power_drv = 0;




    

  display_coupler_values(); 
}
// -------------------------------------------------------------------------------------------------------------------
// ....................................................  line 1 of lcd display  ......................................
void display_coupler_values()
{
  lcd.setCursor(0,0); 
  lcd.print("FWD    W REF   W");


  
  if( power_fwd < 1)
  { dtostrf(power_fwd,2,1, power_fwd_string);            //  variable for converting float type numbers
    lcd.setCursor(3,0);
    lcd.print(power_fwd_string); 
    lcd.setCursor(3,0);
    lcd.print(" ");
  }
  else
  {
  dtostrf(power_fwd,2,0, power_fwd_string);            //  variable for converting float type numbers
  lcd.setCursor(4,0);
  lcd.print(power_fwd_string); 
  }
 
  if(power_ref < 1)   
  { dtostrf(power_ref,2,1, power_ref_string);            //  variable for converting float type numbers
    lcd.setCursor(12,0);
    lcd.print(power_ref_string); 
    lcd.setCursor(12,0);
    lcd.print(" ");
  }
  else
  { dtostrf(power_ref,2,0, power_ref_string);            //  variable for converting float type numbers
    lcd.setCursor(13,0);
    lcd.print(power_ref_string); 
  }    

// ....................................................  line 2 of lcd display  ......................................
  lcd.setCursor(0,1); 
  lcd.print("DRV   mW SWR");                                          
//  lcd.setCursor(6,1);                                           // ****************************************************************

  lcd.setCursor(4,1);
  //if(   (power_drv < power_drv_max) && (power_drv >= power_drv_min)                            )
  { dtostrf(power_drv,2,0, power_drv_string);            //  variable for converting float type numbers ********************-------------------*******3,2
    lcd.print(power_drv_string); 
    lcd.setCursor(2,1); 
    lcd.print("V");
  }

  if(fan_active == YES) {lcd.setCursor(6,1); delay(200); lcd.print(" "); delay(200);}  

  if( error_swr == NO )
  { lcd.setCursor(13,1);        
    reflexion_coefficient = sqrt(power_ref / power_fwd);                       // calculate SWR 

    if(reflexion_coefficient <0) reflexion_coefficient =0; 
    
    if(reflexion_coefficient <= 0.8)
    { swr = (1+reflexion_coefficient)/(1-reflexion_coefficient);
      dtostrf(swr,2,1, swr_string);
      lcd.print(swr_string);                                                   // display SWR
    }
    else  lcd.print(">10");                                                    // SWR too high  
  }
  else
  { lcd.setCursor(13,1);
    lcd.print("ERR");  
  }

  error_swr = NO; 
}
// -------------------------------------------------------------------------------------------------------------------
void menu_temperature() 
{ 
  dtostrf(temperature,4,1, temp_string);  //  variable for converting float type numbers
  lcd.setCursor(0,0);                                                        // colum, line nr.
  lcd.print("PA TEMP.: ");
  lcd.print(temp_string);
  lcd.print("\xDF");                                                         // character sign ° 
  lcd.print("C");
  
// ....................................................  line 2 of lcd display  ......................................
  lcd.setCursor(0,1);
  lcd.print("0____________100");  
  lcd.setCursor(1,1);
  if(temperature<0) temperature=0; if(temperature>100) temperature=100;
  temp_byte = temperature;

  temp_byte /=7.14;                                              // 0 to 100 normalized to0 bis 14
  for(iii=0; iii <= temp_byte; iii++) lcd.print("\xFF");         // \xFF = special character black rectangle

  if(fan_active == YES) {lcd.setCursor(14,0); delay(150); lcd.print(" "); delay(150);}  
  delay(250);       // delay prevents flickering of the display
} 
// -------------------------------------------------------------------------------------------------------------------
void menu_supply_fan()

{ pa_voltage = 0;
  for(iii=0; iii<10; iii++)
  pa_voltage = pa_voltage + analogRead(A4); 
  pa_voltage /= 10;                                               // average of 10 measurements      
                                                               
  pa_voltage = pa_voltage*voltmeter_full_scale/1023;              // adjust R1 when changing voltmeter_full_scale

  lcd.setCursor(0,0); 
  lcd.print("PWR SUPPLY     V");  
  lcd.setCursor(11,0);
  dtostrf(pa_voltage,3,1, pa_voltage_string);                     //  variable for converting float type numbers
  
  lcd.print(pa_voltage_string); 

// ..........................................  line 2 of lcd display  ................................................

  lcd.setCursor(0,1); 
  lcd.print("FAN ON TMP >"); 

  dtostrf(temp_fan_on,2,0, temp_fan_on_string);             //  variable for converting float type numbers to strings
  lcd.print(temp_fan_on_string);

  lcd.print("\xDF"); 
  lcd.print("C");

  if(fan_active == YES) 
  { lcd.setCursor(4,1); delay(300); lcd.print("  "); delay(300); }              // flash word "ON"     
}
// -------------------------------------------------------------------------------------------------------------------
void read_temperature()
{
  if((millis() - time) > 5000)                                          // The sensor is only read out every 5s
  { sensors.requestTemperatures();                                      // because the process takes 750ms
    temperature = sensors.getTempCByIndex(0);                           // in high resolution mode.    
    time = millis(); 

      
    if(temperature >0 && temperature <100)                            // display PA temperature on DRV position for 1s
    { lcd.setCursor(0,1);
      lcd.print("TMP ");
      dtostrf(temperature,2,0, temp_string);                              
      lcd.print(temp_string);
      lcd.print("\xDF");                                                         // character sign ° 
      lcd.print("C");
      delay(1000);
    }     
  } 
  
  if(temperature >= temp_fan_on)   {  digitalWrite(FAN_PIN, HIGH); fan_active = YES; }
    else  {  digitalWrite(FAN_PIN, LOW) ; fan_active =  NO; }      
}
// -------------------------------------------------------------------------------------------------------------------
